/* ftTCP_client.c
 * Client TCP pour le serveur ftTCP_server permettant d'envoyer
 * le contenu d'un fichier au serveur.
 *
 * Auteur: VOURIOT Thierry
 * Web: http://thierry.vouriot.free.fr
 * Email: thierry.vouriot@free.fr
 * Date: 16 octobre 2002
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <fcntl.h>

/* Fonction affichant un message d'erreur en fonction de la valeur
 * de errno et quitte le programme.
 */
void erreur(char *s)
{
	perror(s);
	exit(-1);
}

#define BUFSIZE 100	// taille du buffer
int main(int argc, char **argv)
{
	struct hostent *host;
	struct sockaddr_in addr;	// adresse du serveur
	int sock;			// socket
	int fd;				// descripteur de fichier
	int port;			// port du serveur
	char buf[BUFSIZE];		// buffer
	int nbpackets = 0;		// nombre de packets envoys au serveur
	int nbuf;			// nb de char envoys
	
	/* Test des arguments de la ligne de commande */
	if (argc != 4) {
		printf("Usage: ftTCP_client nommachine port fichier\n");
		exit(0);
	}
	
	/* Ouverture du fichier */
	fd = open(argv[3], O_RDONLY);
	if (fd < 0) erreur("open");
	
	/* Rcupre l'adresse du serveur  partir de son nom */
	host = gethostbyname(argv[1]);
	if (host == NULL) erreur("gethostbyname");
	
	/* Cration du socket */
	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock < 0) erreur("socket");
	
	/* Test de la validit du port */
	port = atoi(argv[2]);
	if (port < 1024 || port > 65000) {
		printf("Numro de port invalide: 1024 < port < 65000\n");
		exit(-1);
	}
	
	/* Cration de la structure sockaddr_in (adresse du serveur) */
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);	// port
	memcpy(&addr.sin_addr, host->h_addr, sizeof(addr.sin_addr));	// copie de l'adresse IP
	memset(&(addr.sin_zero), '\0', 8);	// on met  zro le reste de la struct
	
	/* Connection au serveur */
	if (connect(sock, (struct sockaddr*) &addr, sizeof(struct sockaddr)) < 0) 
		erreur("connect");
	
	/* Boucle pour l'envoie des packets contenant le fichier */
	while ((nbuf = read(fd, buf, BUFSIZE)) > 0) {
		if (send(sock, buf, nbuf, 0) != nbuf) 
			erreur("write");
		nbpackets++;
	}
	
	printf("Nombre de packets envoyes = %d\n", nbpackets);
	
	/* Fermeture des descripteurs et sockets */
	close(fd);
	close(sock);
	
	return 1;
}
